/*****************************************************************
/
/ File   :   ixraw.c
/ Author :   Mrten Norman
/ Date   :   Februari 1999
/ Purpose:   Talks to Intertex Smartcard Modem. Provides the "raw"
/            function IX_CrdIO().
/ License:   See file LICENSE
/
******************************************************************/

#include <stdio.h>
#include "serial.h"
#include "ixapi.h"
#include "ixraw.h"

/*
// Variables local to this file
*/

static char            eventflag = 0; /* Signals reader status change */
static unsigned char   checkbyte;     /* Counts frame checksum */

/*
//  Transmits a byte to modem/reader
//
//  Returns:          1 = transmit error
//                    0 = ok
*/
static int putbyte(char c) {
  if (IO_Write(c) == FALSE) return 1;
  return 0;
}

/*
//  Transmits a byte to modem/reader while fixing DLEs + checkbyte
//
//  In:               c  data to send
//
//  Returns:          1 = transmit error
//                    0 = ok
*/
static int ixputbyte(unsigned char c) {
  checkbyte ^= c;
  if (IO_Write(c) == FALSE) return 1;
  if (c == DLE) { if (IO_Write(c) == FALSE) return 1; }
  return 0;
}

/*
//  Receives a byte from modem/reader
//
//  Returns:          -1      = Receive error/timeout
//                    0..0xff = data
*/
static int getbyte() {
  BYTE buffer;
  if (IO_Read(1, &buffer) == FALSE) return -1;
  return buffer;
}

/*
//  Receives a byte from modem/reader while fixing DLEs + checkbyte
//
//  Returns:          -1      = Receive error/timeout
//                    -10     = IX_BEGIN
//                    -20     = IX_END
//                    0..0xff = data
*/
static int ixgetbyte() {
  BYTE buffer;
  if (IO_Read(1, &buffer) == FALSE) return -1;
  if (buffer == DLE) {
    if (IO_Read(1, &buffer) == FALSE) return -1;

    switch (buffer) {
    case STX:
      return IX_BEGIN; /* Don't count start into checkbyte! */
      break;
    case ETX:
      return IX_END;  /* Don't count end into checkbyte! /*
      break;
    case DC4: /* Status change */
    /* Not supported yet. Should not happen unless asked for... */ 
      break;
    case DLE:  /* Just swallow doubled DLE */
    default:
      break;
    }
  }
  checkbyte ^= buffer;
  return buffer;
}

/*
//  Transmits a string to modem/reader
//
//  All <DLE> etc. stuff is performed here.
//  
//  In:               cmd     command byte
//                    par     parameter byte
//                    len     length count of string
//            string  Data to send. Not NULL-terminated (as C strings are)
//  Returns:          1 = transmit error
//                    0 = ok
//
*/
static int String2Crd(
		      unsigned char cmd,
		      unsigned char par,
		      int len,
		      const unsigned char *string)
{
  checkbyte = 0; /* Reset checksum */

  if (putbyte(DLE)) return 1;
  if (putbyte(STX)) return 1;

  if (ixputbyte(cmd)) return 1;
  if (ixputbyte(par)) return 1;

  while(len-- > 0) {
    if (ixputbyte(*string++)) return 1;
  }

  if (ixputbyte(checkbyte)) return 1;
  if (putbyte(DLE)) return 1;
  if (putbyte(ETX)) return 1;

  return 0;
}

/*
//  Transmits a null terminated string to modem/reader
//
//  Note: 
//
//  In:               string  NULL-terminated data to send
//  Returns:          1 = transmit error
//                    0 = ok
//
*/
int String2Mdm(unsigned char *string) {

  while(*string != '\0') {
    if (putbyte(*string++)) return 1;
  }

  return 0;
}

/*
//  Receives a string from modem/reader.
//
//  All <DLE> etc. stuff is performed here.
//  
//
//  In:               cmd     command byte
//                    par     parameter byte
//                    maxlen  Max length allowed for string.
//                    string  Pointer to inbuffer. Will not be used if
//                            maxlen == 0.
//
//  Returns:  Number of bytes received. 
//            -1 if communication error.
//            -2 if maxlen overflow.
//            -3 if checksum error
*/
static int Crd2String(
	       unsigned char *cmd,
	       unsigned char *par,
	       int maxlen,
	       unsigned char *string)
{
  int c;
  int lastc = -1;
  int len;
  int timeout;

  timeout = 5;     /* Seconds to wait for response */

  for (;;) { /* Wait for begin */
    c = ixgetbyte();
    if (c == -1) { if (--timeout <= 0) return -1; }
    if (c == IX_BEGIN) break;
  }

  checkbyte = 0;   /* Reset checksum */
  len = 0;

  /*
   *  Get <CMD> <PAR> first. They will always be there, so an
   *  IX_END or timeout is an error.
   */
  c = ixgetbyte();  if ( c < 0 || c > 255 ) return -1;
  *cmd = c;
  c = ixgetbyte();  if ( c < 0 || c > 255 ) return -1;
  *par = c;

  for(;;) { /* Get <DATA>, if any */
    c = ixgetbyte();
    if ( c == -1) return -1;
    
    if (c == IX_END) {
      if (checkbyte == 0) {
	ixgetbyte(); ixgetbyte(); /* Swallow <CR> <LF> */
	ixgetbyte(); ixgetbyte(); /* Swallow OK */
	ixgetbyte(); ixgetbyte(); /* Swallow <CR> <LF> */
	return len;   /* OK! return length. */
      }
      else {
	return -3;    // Checksum error!
      }
    }
    else {
      if (lastc == -1) {
	lastc = c; /* Use lastc as a 1-byte delay buffer to avoid */
      }            /* wasting return buffer space on the LRC byte */
      else {
	if (len >= maxlen) return -2; /* Buffer overflow */
	*string++ = lastc;
	lastc = c;
	len++;
      }
    }
  }
}

/*
//  Scans input from modem until CONNICC or timeout
//
//
//  Returns:  1 if CONNICC
//            0 if timeout or error
*/
static int GetConnIcc() {

  static unsigned char icc[]="CONNICC";
  unsigned char *ptr = icc;
  int c;

  do {
    c = getbyte();
    if ( c < 0 ) return 0; /* timeout or error */
    if (*ptr == c) ptr++; else ptr = icc;
  } while (*ptr != '\0');

  return 1;
}


/*
//  Sends a smart card command to the modem/reader and returns result.
//
//  In:               cmdin   command byte
//                    parin   parameter byte
//                    lin     Length of datain
//                    datain  Pointer to datain buffer
//                    lout    Max length allowed for string
//                    datain  Pointer to datain buffer
//
//  Out:              cmd     Echo of command byte
//                    par     Response code
//                    lout    Length of received data
//
//  Returns:          0 if OK
//                    Otherwise error code.
*/
int IX_CrdIO(
	     unsigned char cmdin,
	     unsigned char parin,
	     int lin, 
	     const unsigned char *datain,
	     unsigned char *cmd,
	     unsigned char *par,
	     int *lout,
	     unsigned char *dataout)
{
#ifdef PCSC_DEBUG
  int t;
  printf("--> IX reader: <%3u> <%3u> ", cmdin, parin);
  for (t = 0; t < lin; t++)
    printf("%02x ", datain[t]);
  printf("\n");
#endif

  String2Mdm("at*sc\r");
  if (!GetConnIcc()) return ICC_ERR_ATSCFAIL;
  if (String2Crd(cmdin, parin, lin, datain)) return ICC_ERR_MSGFAIL;
  *lout = Crd2String(cmd, par, *lout, dataout);

#ifdef PCSC_DEBUG
  printf("<-- IX reader: <%3u> <%3u> ", *cmd, *par);
  for (t = 0; t < *lout; t++)
    printf("%02x ", dataout[t]);
  printf("\n***********************************************************\n");
#endif

  if (*lout == -1) return ICC_ERR_MSGFAIL;
  if (*lout == -2) return ICC_ERR_BUFTOOSMALL;
  if (*lout == -3) return ICC_ERR_MSGCORRUPT;
  if (*par >= 128) return *par; /* Return error code */
  return 0; /* OK! */
}

